/*! A contribution to the open-source movement.
 *  Jean-Luc Cooke <jlcooke@certainkey.com>
 *     CertainKey Inc.
 *     Ottawa Ontario Canada
 *
 *  Created: July 20th, 2001
 *
 *  The following program code is released under the GPL license
 *    http://www.gnu.org/copyleft/gpl.html
 */

#include <stdio.h>
#include "sha256.h"

const uns8 ans1[SHA256_DIGEST_SIZE] = {0xba, 0x78, 0x16, 0xbf, 0x8f, 0x01, 0xcf, 0xea, 0x41, 0x41, 0x40, 0xde, 0x5d, 0xae, 0x22, 0x23, 0xb0, 0x03, 0x61, 0xa3, 0x96, 0x17, 0x7a, 0x9c, 0xb4, 0x10, 0xff, 0x61, 0xf2, 0x00, 0x15, 0xad,};
const uns8 ans2[SHA256_DIGEST_SIZE] = {0x24, 0x8d, 0x6a, 0x61, 0xd2, 0x06, 0x38, 0xb8, 0xe5, 0xc0, 0x26, 0x93, 0x0c, 0x3e, 0x60, 0x39, 0xa3, 0x3c, 0xe4, 0x59, 0x64, 0xff, 0x21, 0x67, 0xf6, 0xec, 0xed, 0xd4, 0x19, 0xdb, 0x06, 0xc1};
const uns8 ans3[SHA256_DIGEST_SIZE] = {0xcd, 0xc7, 0x6e, 0x5c, 0x99, 0x14, 0xfb, 0x92, 0x81, 0xa1, 0xc7, 0xe2, 0x84, 0xd7, 0x3e, 0x67, 0xf1, 0x80, 0x9a, 0x48, 0xa4, 0x97, 0x20, 0x0e, 0x04, 0x6d, 0x39, 0xcc, 0xc7, 0x11, 0x2c, 0xd0};

int main() {
  char *test1 = "abc";
  char *test2 = "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq";
  uns8 D[SHA256_DIGEST_SIZE];
  sha256_context CTX;
  int i;

  printf("sha256(%s)\n", test1);
  /* strlen < 56 */
  sha256_init(&CTX);
  sha256_update(&CTX, (uns8*)test1, strlen(test1));
  sha256_final(D, &CTX);

  printf("D = "); for (i=0; i<SHA256_DIGEST_SIZE; i++) printf("%.2x ", D[i]); printf("\n");
  printf("A = "); for (i=0; i<SHA256_DIGEST_SIZE; i++) printf("%.2x ", ans1[i]); printf("\n");


  /* strlen == 56 */
  printf("sha256(%s)\n", test2);
  sha256_init(&CTX);
  sha256_update(&CTX, (uns8*)test2, strlen(test2));
  sha256_final(D, &CTX);

  printf("D = "); for (i=0; i<SHA256_DIGEST_SIZE; i++) printf("%.2x ", D[i]); printf("\n");
  printf("A = "); for (i=0; i<SHA256_DIGEST_SIZE; i++) printf("%.2x ", ans2[i]); printf("\n");


  /* strlen == 56 */
  printf("sha256(%s)\n", test2);
  sha256_init(&CTX);
  sha256_update(&CTX, (uns8*)test2, strlen(test2)/2);
  sha256_update(&CTX, (uns8*)&test2[strlen(test2)/2], strlen(test2) - strlen(test2)/2);
  sha256_final(D, &CTX);

  printf("D = "); for (i=0; i<SHA256_DIGEST_SIZE; i++) printf("%.2x ", D[i]); printf("\n");
  printf("A = "); for (i=0; i<SHA256_DIGEST_SIZE; i++) printf("%.2x ", ans2[i]); printf("\n");


  /* crazy long! */
  printf("sha256(1,000,000 x 'a')\n");
  sha256_init(&CTX);
  for (i=0; i<10*1000; i++)
    sha256_update(&CTX, (uns8*)"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", 100);
  sha256_final(D, &CTX);

  printf("D = "); for (i=0; i<SHA256_DIGEST_SIZE; i++) printf("%.2x ", D[i]); printf("\n");
  printf("A = "); for (i=0; i<SHA256_DIGEST_SIZE; i++) printf("%.2x ", ans3[i]); printf("\n");

  
  return 0;
}
